/*
 *  Free software to write a security pattern over deleted file-space
 *  in the wee hours of the night.
 *  cron usage:
 *  security /filename
 *  security /home/filename
 *  security /tmp/filename
 *
 *  This writes a file large enough to fill up the disk partition,
 *  syncs the file-system, then deletes the file.
 *  Created 20-JUL-1998 RichardBJohnson@comcast.net
 *
 *  No Copyright is claimed and there is no warranty.
 */


#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>

#ifdef _MSC_VER
#include <io.h>
void sync(void){}
#else
#include <unistd.h>
#endif

#define BUF_LEN 0x10000
#define FILENAME argv[1]
#define WHO argv[0]
#define LEN (sizeof(secur) -1)

#if !defined(O_BINARY)
#define O_BINARY 0
#endif
#if !defined(O_SYNC)
#define O_SYNC 0
#endif

#define ArraySize(a) (sizeof(a) / sizeof(a[0]))

static void fill_buffer_text(char *buf, size_t len);
static void fill_buffer_rand(char *buf, size_t len);
static void fill_buffer_blnk(char *buf, size_t len);
static void fill_buffer_invt(char *buf, size_t len);
struct {
	void (*setup)(char *, size_t);
	const char *text;
} todo[] = {
	{ fill_buffer_text, "text"          },
	{ fill_buffer_rand, "random"        },
	{ fill_buffer_blnk, "blanks"        },
	{ fill_buffer_invt, "inverted text" },
	{ fill_buffer_rand, "random"        },
	{ fill_buffer_blnk, "blanks"        },
};

const char secur[]="SECURITY        ";

//
// Fill buffer with text
//
static void fill_buffer_text(char *buf, size_t len)
{
	size_t i;
    for(i=0; i< len / LEN; i++)
    {
        memcpy(buf, secur, LEN);
        buf += LEN;
    }
}
//
//  Fill buffer with random junk
//
static void fill_buffer_rand(char *buf, size_t len)
{
	size_t i;
	int *wrd = (int *) buf;
    for(i=0; i< len / sizeof(int); i++)
		*wrd++ = rand();
}
//
//  Fill buffer with blanks
//
static void fill_buffer_blnk(char *buf, size_t len)
{
    memset(buf, 0x20, len);
}
//
//  Fill buffer witj inverted text
//
static void fill_buffer_invt(char *buf, size_t len)
{
    fill_buffer_text(buf, len);
	while(len--)
		*buf++ ^= 0xff;

}

int main(int args, char *argv[])
{
    int fd;
	size_t i;
    char *buffer;

    if(args < 2)
    {
        fprintf(stderr, "Usage:\n");
        fprintf(stderr, "%s pathname\n", WHO);
		FILENAME = "C:\\SECURE.DAT";
    }
    if((buffer = (char *) malloc(BUF_LEN *sizeof(char))) == NULL)
    {
        fprintf(stderr, "From %s, can't allocate memory\n", WHO);
        exit(EXIT_FAILURE);
    }
	for(i = 0; i < ArraySize(todo); i++)
	{
        printf("Writing %s\n", todo[i].text);
		(*todo[i].setup)(buffer, BUF_LEN);

        if((fd = open(FILENAME, O_RDWR|O_CREAT|O_TRUNC|O_SYNC|O_BINARY, 0644)) < 0)
		{
            fprintf(stderr, "From %s, file creation error %s\n", WHO,
               strerror(errno));
			free(buffer);
            exit(EXIT_FAILURE);
		}		
        for(;;)
		{
            if(write(fd, buffer, BUF_LEN) != BUF_LEN)
                break;
		}
        (void)close(fd);
        (void)sync();
        (void)unlink(FILENAME);
	}
	free(buffer);
    return EXIT_SUCCESS;
}


